<?php

namespace ProjectManagers\ProjectManager\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use ProjectManagers\ProjectManager\Forms\Manager as ManagerForm;
use ProjectManagers\ProjectManager\Http\Requests\ManagerRequest;
use ProjectManagers\ProjectManager\Interfaces\ManagerRepositoryInterface;
use ProjectManagers\ProjectManager\Repositories\Eloquent\Filters\ManagerResourceFilter;
use ProjectManagers\ProjectManager\Repositories\Eloquent\Presenters\ManagerListPresenter;

/**
 * Resource controller class for manager.
 */
class ManagerResourceController extends BaseController
{

    /**
     * Initialize manager resource controller.
     *
     *
     * @return null
     */
    public function __construct(ManagerRepositoryInterface $manager)
    {
        parent::__construct();
        $this->form = ManagerForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('project_managers.project_manager.modules'), 'project_manager', guard_url('project_manager'));
        $this->repository = $manager;
    }

    /**
     * Display a list of manager.
     *
     * @return Response
     */
    public function index(ManagerRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(ManagerResourceFilter::class)
            ->setPresenter(ManagerListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('project_manager::manager.names'))
            ->view('project_manager::manager.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display manager.
     *
     * @param Request $request
     * @param Model   $manager
     *
     * @return Response
     */
    public function show(ManagerRequest $request, ManagerRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('project_manager::manager.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('project_manager::manager.show')
            ->output();
    }

    /**
     * Show the form for creating a new manager.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(ManagerRequest $request, ManagerRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('project_manager::manager.name'))
            ->view('project_manager::manager.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new manager.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ManagerRequest $request, ManagerRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('project_manager::manager.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('project_manager/manager/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/project_manager/manager'))
                ->redirect();
        }

    }

    /**
     * Show manager for editing.
     *
     * @param Request $request
     * @param Model   $manager
     *
     * @return Response
     */
    public function edit(ManagerRequest $request, ManagerRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('project_manager::manager.name'))
            ->view('project_manager::manager.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the manager.
     *
     * @param Request $request
     * @param Model   $manager
     *
     * @return Response
     */
    public function update(ManagerRequest $request, ManagerRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('project_manager::manager.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('project_manager/manager/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('project_manager/manager/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the manager.
     *
     * @param Model   $manager
     *
     * @return Response
     */
    public function destroy(ManagerRequest $request, ManagerRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('project_manager::manager.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('project_manager/manager/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('project_manager/manager/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
